#!/usr/bin/perl

#retrieve (if we can) and process the following files

#ftp://ftp.arin.net/netinfo/asn.txt
#ftp://ftp.apnic.net/pub/apnic/dbase/data/apnic.an.gz
#ftp://ftp.ripe.net/ripe/dbase/split/ripe.db.an.gz

#and put them all in another file in a dumb format so they can
#be processed by my dumb C program.

#By the way, this program is also very dumb.

undef $MissingFile;
if (! -f "asn.txt" ) { 
	print "Cannot find asn.txt.  Attempting to retrieve.\n";
	if (! RetrieveFile("ftp://ftp.arin.net/netinfo/asn.txt","asn.txt")) {
		print "---> Cannot retrieve asn.txt\n";
		print "---> Please download it from\n";
		print "---> ftp://ftp.arin.net/netinfo/asn.txt\n";
		$MissingFile=1;
	}
}

$gzip="gzip";

if (! -f "apnic.an" ) { 
	if (-f "apnic.an.gz") {
		system "$gzip -d apnic.an.gz";
		$r=$?;
		if ($r > 0) {
			print "---> Retrieved apnic.an.gz, but could not run $gzip!\n";
			print "---> Please get $gzip and try again\n";
			exit(1);
		}
	} else { 
		if ( RetrieveFile("ftp://ftp.apnic.net/pub/apnic/dbase/data/apnic.an.gz","apnic.an.gz")) {
			system "$gzip -d apnic.an.gz";
			$r=$?;
			if ($r > 0) {
				print "---> Retrieved apnic.an.gz, but could not run $gzip!\n";
				print "---> Please get $gzip and try again\n";
				exit(1);
			}
		} else {
			print "---> Cannot retrieve apnic.an.gz\n";
			print "---> Please download it from\n";
			print "---> ftp://ftp.apnic.net/pub/apnic/dbase/data/apnic.an.gz\n";
			$MissingFile=1;
		}
	}
}

if (! -f "ripe.db.an" ) { 
	if (-f "ripe.db.an.gz") {
		system "$gzip -d ripe.db.an.gz";
		$r=$?;
		if ($r > 0) {
			print "---> Retrieved ripe.db.an.gz, but could not run $gzip!\n";
			print "---> Please get $gzip and try again\n";
			exit(1);
		}
	} else { 
		if ( RetrieveFile("ftp://ftp.ripe.net/ripe/dbase/split/ripe.db.an.gz","ripe.db.an.gz")) {
			system "$gzip -d ripe.db.an.gz";
			$r=$?;
			if ($r > 0) {
				print "---> Retrieved ripe.db.an.gz, but could not run $gzip!\n";
				print "---> Please get $gzip and try again\n";
				exit(1);
			}
		} else {
			print "---> Cannot retrieve ripe.db.an.gz\n";
			print "---> Please download it from\n";
			print "---> ftp://ftp.ripe.net/ripe/dbase/split/ripe.db.an.gz\n";
			$MissingFile=1;
		}
	}
}

if ($MissingFile) {
	print "\n---> Please get the file(s) listed above and rerun ProcessASN.pl or Make.\n";
	exit(1);
}

#process asn.txt

$c=0;
open ASN,"<asn.txt";
while (<ASN>) {
	chomp;
	next if (/^  *$/);
	next if (/^\[/);
	next if (/^[a-zA-Z]/);

	if (/^  *([0-9][0-9]*)  *([a-zA-Z0-9\-][a-zA-Z0-9\-]*)  *(\[..*\])$/) {
		$ASNs{$1}=$2;
		$c++;
	}
	elsif (/^  *([0-9][0-9]*)\-([0-9][0-9]*)  *([a-zA-Z0-9\-][a-zA-Z0-9\-]*)  *(\[..*\])$/) {
		for ($i=$1; $i <= $2; $i++) {
			$ASNs{$i}=$3;
			$c++;
		}
	}
}

close ASN;
print "Processed $c ASes from asn.txt\n";

ProcessRipeDB("apnic.an");
ProcessRipeDB("ripe.db.an");

$c=0;
open OUTPUT,">asnc.txt";
foreach $k (sort {$a <=> $b } (keys %ASNs)) {
  print OUTPUT "$k $ASNs{$k}\n";
  $c++;
}
print "Wrote $c ASes to asnc.txt.  Bye.\n";

sub ProcessRipeDB {
  $file=shift @_;
  $c=0;
  open APNIC,"<$file";
  while (<APNIC>) {
	if (/^\*an: AS(..*)/) {
		if ($an) {
			if ($aa) { $ASNs{$an}=$aa; $c++; }
			elsif ($mb) { $ASNs{$an}=$mb; $c++; }
			elsif ($de) { $ASNs{$an}= $de; $c++; }
			else { print "AS $an missing a name!\n"; }
			
			undef $an; undef $aa; undef $mb; undef $de;
		}
		$an=$1;
	}
	if (/^\*aa: (..*)/) {
		if (! $aa) {
			$aa=$1;
			$aa=~ s/ /_/g;
			if ($aa eq 'UNSPECIFIED') {
			    $aa="AS$an"; # 'an:' is always defined here
			}
		}
	} 
	if (/^\*mb: MAINT\-(..*)/) {
		if (! $mb) {
			$mb=$1;
			$mb=~ s/ /_/g;
		}
	} elsif (/^\*mb: (..*)/) {
		if (! $mb) {
			$mb=$1;
			$mb=~ s/ /_/g;
		}
	} 
	if (/^\*de: (..*) autonomous system/) {
		if (! $de) {
			$de=$1;
			$de=~ s/ /_/g;
		}
	} elsif (/^\*de: (..*)/) {
		if (! $de) {
			$de=$1;
			$de=~ s/ /_/g;
		}
	}
  }
  print "Processed $c ASes from $file\n";
}

sub RetrieveFile {
  my $location=shift @_;
  my $filename=shift @_;

  print "RetrieveFile: getting $location\n";

  $wget="wget";
  $ncftp="ncftp";
  $curl="curl";
  system "$wget -q $location";
  $r=$?;
  if ($r==0) { print "Got file with $wget.\n"; return 1; }

  system "$ncftp $location";
  $r=$?;
  if ($r==0) { print "Got file with $ncftp.\n"; return 1; }

  system "$curl -o $filename $location";
  $r=$?;
  if ($r==0) { print "Got file with $curl\n"; return 1; }
  if ($r > 0) { print "Could not get $filename with $curl, $wget or $ncftp.\n"; }
  return 0;
}
