/*****************************************************************************
* Copyright (C) 2000 Nik Weidenbacher
* nikw@martnet.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*****************************************************************************/

#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
/*#include <glib.h> */
#include <sys/ioctl.h>
#include <termios.h>
#include <unistd.h>
#include <signal.h>
#include "netflowv5.h"

struct ehnt_config {
  char servername[100];
  int serverport;
  int mode;
  int topmode;
  char server[100];
  int zero_as;
  int flowlimit;
  int debug;
  int si; /* sampling interval (actually, reporting interval) */
  int screenlength;
};

struct ehnt_filter {
  int AS;
  int port;
  int proto;
  int big;
  int intnum;
  struct in_addr *router;
  struct in_addr *prefix;
  struct in_addr *pfxmask;
};

struct ehnt_stats {
  double InBoundTotal, OutBoundTotal;
  double *InBoundSrcAS, *InBoundDstAS, *OutBoundSrcAS, *OutBoundDstAS;
  double *InBoundProto, *OutBoundProto;
  double *InBoundSrcTcpPort, *InBoundDstTcpPort, 
    *OutBoundSrcTcpPort, *OutBoundDstTcpPort;
  double *InBoundSrcUdpPort, *InBoundDstUdpPort, 
    *OutBoundSrcUdpPort, *OutBoundDstUdpPort;
  /*
  GHashTable * InBoundSrcIP, *InBoundDstIP, *OutBoundSrcIP, *OutBoundDstIP;
  GHashTable * InBoundSrcPfx, *InBoundDstPfx, *OutBoundSrcPfx, *OutBoundDstPfx;
  */
  time_t interval, time;
};

struct ehnt_data {
  int recentmin;
  struct ehnt_stats * stats;
  struct ehnt_stats * old_stats;
  time_t last_t;
  int c,i;
  double flowcount,packetcount;
  int maxbytesAS1, maxbytesAS2, maxpacketsAS1, maxpacketsAS2;
  double maxbytes, maxpackets;
};

struct ehnt_struct {
  struct ehnt_config * cfg;
  struct ehnt_filter * flt;
  struct ehnt_data * dat;
};

int ehnt_client(char *, int, struct ehnt_struct *);
int ProcessFlow(struct flow_ver5_rec *, struct in_addr *, struct ehnt_struct *);
int Init_ASN_Lookups(void);
char * ASN_Lookup(int);
char * prettybytes(int);
void DisplayReport(time_t, struct ehnt_data *);

/* these EM_ defines are for the mode config variable */
#define EM_TOP    1
#define EM_DUMP   2
#define EM_SHORTDUMP 3
#define EM_COLONDUMP 4

/* the ETM_ defines are for the topmode config variable */
#define ETM_AS        1
#define ETM_PROTO     2
#define ETM_TCPPORT   3
#define ETM_UDPPORT   4

/* AS table size */
#define AS_TS  (65536*sizeof(double))
/* Proto table size */
#define PROTO_TS  (65536*sizeof(double))
/* TcpPort table size */
#define TCPPORT_TS  (65536*sizeof(double))
/* UdpPort table size */
#define UDPPORT_TS  (65536*sizeof(double))

#define SET_TERM_RAW 1
#define SET_TERM_COOKED 2
