/*****************************************************************************
* Copyright (C) 2000 Nik Weidenbacher
* nikw@martnet.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*****************************************************************************/

#include <netdb.h>
#include "config.h"
#include "ehnt.h"

#define uc (unsigned char)
#define MAXPACKET 4096

static void sig_int (int);
char getkey();

int ehnt_client(char * servername, int serverport, struct ehnt_struct *my_ehnt_struct ) {
 char ips[100];
  int c;
 
 int sockfd;
 int recordsize,headersize;
 struct sockaddr_in      servaddr, cliaddr;
 socklen_t len;
 struct hostent * h_server;
 struct ehnt_config * e_cfg=my_ehnt_struct->cfg;

 if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
   { perror("socket error"); exit(0); }

 memset(&servaddr,0,sizeof(servaddr));
 servaddr.sin_family      = AF_INET;
 if ( (h_server=gethostbyname(servername)) == NULL) {
#ifdef LINUX
   herror("gethostbyname error");
#else
   perror("gethostbyname error");
#endif
   exit(0); 
 }
 memcpy(&servaddr.sin_addr.s_addr,h_server->h_addr_list[0],4);
 servaddr.sin_port        = htons(serverport);

 if ( connect(sockfd,(struct sockaddr *) &servaddr, sizeof(servaddr)) < 0)
   { perror("connect error"); exit(1); }

 {
   int one=1;
   if (setsockopt(sockfd, SOL_SOCKET, SO_KEEPALIVE,(char *)&one, sizeof one) < 0) {
     perror("setsockopt(rfd, KEEPALIVE):%s");
     exit(1);
   }
 }

 if (signal(SIGINT,sig_int) == SIG_ERR) {
   printf("Warning: can't catch interrupt signal\n");
 }
 fflush(stdout);
 set_term(SET_TERM_RAW);
 setbuf(stdout,NULL);

 for ( ; ; ) {
   struct in_addr routeraddr;
   fd_set read_fds;
   int maxfd=sockfd;
   
   FD_ZERO(&read_fds);
   FD_SET(STDIN_FILENO,&read_fds);
   FD_SET(sockfd,&read_fds);

   fflush(stdout);
   if ( (c=select(maxfd+1,&read_fds,NULL,NULL,NULL)) < 0) {
     printf("select (read) error"); exit(0);
   }

   if (FD_ISSET(sockfd,&read_fds)) {
     ehnt_client_processmessage(sockfd,routeraddr,my_ehnt_struct);
   }
   if (FD_ISSET(STDIN_FILENO,&read_fds)) {
     char key;
     key=getkey();
     switch (key) {
     case 'a': /* AS report */
       e_cfg->topmode=ETM_AS;
       ShowReport(my_ehnt_struct->dat->old_stats,my_ehnt_struct);
       break;
     case 'p': /* proto report */
       e_cfg->topmode=ETM_PROTO;
       ShowReport(my_ehnt_struct->dat->old_stats,my_ehnt_struct);
       break;
     case 't': /* tcp report */
       e_cfg->topmode=ETM_TCPPORT;
       ShowReport(my_ehnt_struct->dat->old_stats,my_ehnt_struct);
       break;
     case 'u': /* udp report */
       e_cfg->topmode=ETM_UDPPORT;
       ShowReport(my_ehnt_struct->dat->old_stats,my_ehnt_struct);
       break;
     default:
     case '?': /* help */
       printf("\rKeys: (a)AS, (p)proto, (t)tcp, (u)udp, (q)uit                             \r");
       break;
     case 3: /* Control-C */
     case 'q': /* quit */
       printf("\rQuitting...                                                               \n");
       set_term(SET_TERM_COOKED);
       exit(0);
       break;
     }
   }

 } /* for ( ; ; ) */
}

int ehnt_client_processmessage(int sockfd,struct in_addr routeraddr, struct ehnt_struct *my_ehnt_struct ) {
  int c,i;
  char mesg[MAXPACKET];
  struct flow_ver5_hdr * flowhdr;
  struct flow_ver5_rec * flowrec;
  char *junk;
  int debug, zero_as;

  debug=my_ehnt_struct->cfg->debug;
  zero_as=my_ehnt_struct->cfg->zero_as;

  /* read src ip address and header */
  c=read_n(sockfd,mesg,sizeof(struct in_addr) + sizeof(struct flow_ver5_hdr));
  if (c == 0) { printf("connection closed\n"); exit(0); }
  if (c <0 ) { perror("read_n"); }
  
  { /* copy the (network-order) router address into a handy variable, routeraddr. */
    memset(&routeraddr,0,sizeof(routeraddr));
    memcpy(&routeraddr.s_addr,&mesg[0],sizeof(routeraddr.s_addr));
  }
  
  flowhdr=(struct flow_ver5_hdr *)((void *)mesg + sizeof(struct in_addr));
  /* change header from network to host byte order */
  flowhdr->version=ntohs(flowhdr->version);
  flowhdr->count=ntohs(flowhdr->count);
  flowhdr->uptime=ntohl(flowhdr->uptime);
  flowhdr->unix_secs=ntohl(flowhdr->unix_secs);
  flowhdr->unix_nsecs=ntohl(flowhdr->unix_nsecs);
  flowhdr->flow_sequence=ntohl(flowhdr->flow_sequence);
  
  if (debug) printf("%d flows in packet\n",flowhdr->count);
  if (flowhdr->version != 5) { 
    printf("Bad netflow packet.  Version is %d, should be 5.\n",flowhdr->version);
    return;
  }
  if ( (flowhdr->count <1) || (flowhdr->count>1000)) {
    printf("Strange netflow packet.  Record count is %d\n",flowhdr->count);
    return;
  }
  
  c=read_n(sockfd,((void *)flowhdr + sizeof(struct flow_ver5_hdr)),
	   flowhdr->count * sizeof(struct flow_ver5_rec));
  if (c < 0 ) { perror("read_n"); exit(0); }
  if (debug) printf("read %d bytes of records\n",c);
  
  if (debug){
    junk=(char *) flowhdr;
    printf("Header debug:\n");
    for (c=0; c<23; c+=12) {
      printf("%3.d: %3.d %3.d %3.d %3.d | %3.d %3.d %3.d %3.d | %3.d %3.d %3.d %3.d\n",c,
	     uc junk[c],uc junk[c+1],uc junk[c+2],uc junk[c+3],
	     uc junk[c+4],uc junk[c+5],uc junk[c+6],uc junk[c+7],
	     uc junk[c+8],uc junk[c+9],uc junk[c+10],uc junk[c+11]);
    }
  }
  
  my_ehnt_struct->dat->packetcount++;
  
  flowrec=(struct flow_ver5_rec *)(flowhdr + sizeof(struct flow_ver5_hdr));
  for (i=0; i<flowhdr->count; i++) {
    my_ehnt_struct->dat->flowcount++;
    
    flowrec=(struct flow_ver5_rec *)(((void *)flowhdr + sizeof(struct flow_ver5_hdr))
    				     + (i*sizeof(struct flow_ver5_rec)));
    
    /* change record from network to host byte order */
    flowrec->input_index=ntohs(flowrec->input_index);
    flowrec->output_index=ntohs(flowrec->output_index);
    flowrec->dPkts=ntohl(flowrec->dPkts);
    flowrec->dOctets=ntohl(flowrec->dOctets);
    flowrec->First=ntohl(flowrec->First);
    flowrec->Last=ntohl(flowrec->Last);
    flowrec->srcport=ntohs(flowrec->srcport);
    flowrec->dstport=ntohs(flowrec->dstport);
    flowrec->src_as=ntohs(flowrec->src_as);
    flowrec->dst_as=ntohs(flowrec->dst_as);

    if (flowrec->dst_as==0) { flowrec->dst_as=zero_as; }
    if (flowrec->src_as==0) { flowrec->src_as=zero_as; }

    ProcessFlow(flowrec,&routeraddr,my_ehnt_struct);

    if (debug){
    junk=(char *) flowrec;
    printf("Record debug:\n");
    for (c=0; c<47; c+=12) {
	printf("%3.d: %3.d %3.d %3.d %3.d | %3.d %3.d %3.d %3.d | %3.d %3.d %3.d %3.d\n",c,
		uc junk[c],uc junk[c+1],uc junk[c+2],uc junk[c+3],
		uc junk[c+4],uc junk[c+5],uc junk[c+6],uc junk[c+7],
		uc junk[c+8],uc junk[c+9],uc junk[c+10],uc junk[c+11]);
	}
   } /* if (debug) */
  } /* for each flow record */
}


/* read n bytes from an fd, don't return until all n bytes are read.
   It is slightly possible that this was partially borrowed from Stevens'
   Unix Network Programming book.
 */
ssize_t read_n(int fd, void *vptr, size_t n) 
{
  size_t  nleft;
  ssize_t nread;
  char    *ptr;

  ptr = vptr;
  nleft = n;
  while (nleft > 0) {
    if ( (nread = read(fd, ptr, nleft)) < 0) {
      if (errno == EINTR)
	nread = 0;
      else
	return(-1);
    } else if (nread == 0)
      break;
    
    nleft -= nread;
    ptr   += nread;
  }
  return(n - nleft);
}

char getkey() {
  char buf[100];
  int a;
  fflush(NULL);
  a=read(0,buf,1);
  
  return buf[0];
}

int set_term(int action) {
  static struct termios tbufsave;

  switch (action) {
  case SET_TERM_RAW:
    {
      struct termios tbuf;

      /*       if (ioctl (STDIN_FILENO, TCGETA, &tbuf) == -1) */
      if (tcgetattr(STDIN_FILENO, &tbuf) == -1)
	printf("ioctl\n");
      tbufsave=tbuf;
      tbuf.c_lflag &= ~(ICANON | ISIG | ECHO );
      tbuf.c_cc[4] = 1;
      tbuf.c_cc[5]=2;
      if (tcsetattr(STDIN_FILENO,TCSANOW,&tbuf) == -1)
	printf("ioctl2\n");
    }
    break;
  case SET_TERM_COOKED:
    tcsetattr(STDIN_FILENO,TCSANOW,&tbufsave);
    break;
  }
}

static void sig_int (int signo) {
  if (signo == SIGINT) {
    printf("Received interrupt.  Exiting.\n");
    set_term(SET_TERM_COOKED);
    exit(0);
  }
  return;
}
