/*****************************************************************************
* Copyright (C) 2000 Nik Weidenbacher
* nikw@martnet.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*****************************************************************************/

#include <string.h>
#include <netdb.h>
#include "config.h"
#include "ehnt.h"

double ShowTopAS (double a[],char t[], int ,double);
double ShowTopProto (double a[], char t[], int ,double);
double ShowTopUdpPort (double a[],char t[], int, double);
double ShowTopTcpPort (double a[],char t[], int, double);
double GetTop (double a[], char t[]);
int ShowValue(char *, int, double, int, double);
int ShowReport(struct ehnt_stats *, struct ehnt_struct *);

int ShowReport(struct ehnt_stats * stats, struct ehnt_struct * my_ehnt_struct) {
  struct ehnt_config * e_cfg=my_ehnt_struct->cfg;
  struct ehnt_filter * e_flt=my_ehnt_struct->flt;
  int report_inout=0;
  time_t interval;
  char *InBoundProtoTags, *OutBoundProtoTags;
  char *InBoundSrcAsTags, *InBoundDstAsTags, *OutBoundSrcAsTags, *OutBoundDstAsTags;
  char *InBoundSrcTcpPortTags, *InBoundDstTcpPortTags, 
    *OutBoundSrcTcpPortTags, *OutBoundDstTcpPortTags;
  char *InBoundSrcUdpPortTags, *InBoundDstUdpPortTags,
    *OutBoundSrcUdpPortTags, *OutBoundDstUdpPortTags;
  int i;
  int headersize=5;
  char header[100];

  if (! stats) {
    printf("No stats to display.  As least one report interval needs to pass        \r");
    return;
  }
  
  interval=stats->interval;

  if ( e_flt->router && e_flt->intnum ) {
    report_inout=1;
  }

  /* Note: DisplayReport destroys the couters for the ASes it displays. */
  switch (e_cfg->topmode) {
  default:
    printf("Report interval expired, but I don't know what type of report to generate!\n");
    printf("Defaulting to AS report\n");
  case ETM_AS:
    InBoundSrcAsTags=malloc(65536);
    InBoundDstAsTags=malloc(65536);
    OutBoundSrcAsTags=malloc(65536);
    OutBoundDstAsTags=malloc(65536);
    memset(InBoundSrcAsTags,0,65536);
    memset(InBoundDstAsTags,0,65536);
    memset(OutBoundSrcAsTags,0,65536);
    memset(OutBoundDstAsTags,0,65536);
    sprintf(header,"    |----AS Name/Num----|-bw;%% of total-| |----AS Name/Num----|-bw,%% of total-|\n");
    headersize=6;
    break;
  case ETM_PROTO:
    InBoundProtoTags=malloc(65536);
    OutBoundProtoTags=malloc(65536);
    memset(InBoundProtoTags,0,65536);
    memset(OutBoundProtoTags,0,65536);
    headersize=4;
    sprintf(header,"Rank|--Proto Name/Num---|-bw;%% of total-|\n");
    break;
  case ETM_TCPPORT:
    InBoundSrcTcpPortTags=malloc(65536);
    InBoundDstTcpPortTags=malloc(65536);
    OutBoundSrcTcpPortTags=malloc(65536);
    OutBoundDstTcpPortTags=malloc(65536);
    memset(InBoundSrcTcpPortTags,0,65536);
    memset(InBoundDstTcpPortTags,0,65536);
    memset(OutBoundSrcTcpPortTags,0,65536);
    memset(OutBoundDstTcpPortTags,0,65536);
    headersize=6;
    sprintf(header,"    |-----TCP Port------|-bw;%% of total-| |-----TCP Port------|-bw,%% of total-|\n");
    break;
  case ETM_UDPPORT:
    InBoundSrcUdpPortTags=malloc(65536);
    InBoundDstUdpPortTags=malloc(65536);
    OutBoundSrcUdpPortTags=malloc(65536);
    OutBoundDstUdpPortTags=malloc(65536);
    memset(InBoundSrcUdpPortTags,0,65536);
    memset(InBoundDstUdpPortTags,0,65536);
    memset(OutBoundSrcUdpPortTags,0,65536);
    memset(OutBoundDstUdpPortTags,0,65536);
    headersize=6;
    sprintf (header,"    |-----UDP Port------|-bw;%% of total-| |-----UDP Port------|-bw,%% of total-|\n");
    break;
  }

  ShowReportTimeHeader(stats);

  if ( ! report_inout ) {
    printf (" Total %10.0fkbits/s\n",stats->InBoundTotal*8/((interval)*1024));
    if ( e_cfg->topmode != ETM_PROTO ) {
      printf ("Rank|-------------Source----------------| |---------------Dest----------------|\n");
    }
    printf("%s",header);

    for (i=1; i<=(e_cfg->screenlength - headersize); i++) {
      printf("%2d:",i);

      switch (e_cfg->topmode) {
      default:
      case ETM_AS:    
	ShowTopAS(stats->InBoundSrcAS,InBoundSrcAsTags,interval,stats->InBoundTotal);
	printf(" ");
	ShowTopAS(stats->InBoundDstAS,InBoundDstAsTags,interval,stats->InBoundTotal);
	break;
      case ETM_PROTO:
	ShowTopProto(stats->InBoundProto,InBoundProtoTags,interval,stats->InBoundTotal);
	break;
      case ETM_TCPPORT:
	ShowTopTcpPort(stats->InBoundSrcTcpPort,InBoundSrcTcpPortTags,
		       interval,stats->InBoundTotal);
	printf(" ");
	ShowTopTcpPort(stats->InBoundDstTcpPort,InBoundDstTcpPortTags,
		       interval,stats->InBoundTotal);
	break;
      case ETM_UDPPORT:
	ShowTopUdpPort(stats->InBoundSrcUdpPort,InBoundSrcUdpPortTags,
		       interval,stats->InBoundTotal);
	printf(" ");
	ShowTopUdpPort(stats->InBoundDstUdpPort,InBoundDstUdpPortTags,
		       interval,stats->InBoundTotal);
	break;
      }
      printf("\n");
    }
  } else {
    if ( e_cfg->topmode != ETM_PROTO ) {
      printf ("Rank|-------------Source----------------| |---------------Dest----------------|\n");
    }
    printf("%s",header);
    printf ("                          O   U   T   B   O   U   N   D  (total %.0fkbits/s)\n",
	    stats->OutBoundTotal*8/((interval)*1024));

    for (i=1; i<= (int)(e_cfg->screenlength - headersize)/2 + 1; i++) {
      printf("%2d:",i);
      switch (e_cfg->topmode) {
      default:
      case ETM_AS:
	ShowTopAS(stats->OutBoundSrcAS,OutBoundSrcAsTags,interval,stats->OutBoundTotal);
	printf(" ");
	ShowTopAS(stats->OutBoundDstAS,OutBoundDstAsTags,interval,stats->OutBoundTotal);
	break;
      case ETM_PROTO:
	ShowTopProto(stats->OutBoundProto,OutBoundProtoTags,interval,stats->OutBoundTotal);
	break;
      case ETM_TCPPORT:
	ShowTopTcpPort(stats->OutBoundSrcTcpPort,OutBoundSrcTcpPortTags,
		       interval,stats->OutBoundTotal);
	printf(" ");
	ShowTopTcpPort(stats->OutBoundDstTcpPort,OutBoundDstTcpPortTags,
		       interval,stats->OutBoundTotal);
	break;
      case ETM_UDPPORT:
	ShowTopUdpPort(stats->OutBoundSrcUdpPort,OutBoundSrcUdpPortTags,
		       interval,stats->OutBoundTotal);
	printf(" ");
	ShowTopUdpPort(stats->OutBoundDstUdpPort,OutBoundDstUdpPortTags,
		       interval,stats->OutBoundTotal);
	break;
      }
      printf("\n");
    }
    printf ("                            I   N   B   O   U   N   D    (total %.0fkbits/s)\n",
	    stats->InBoundTotal*8/((interval)*1000));
    for (i=1; i<= (int)(e_cfg->screenlength - headersize)/2; i++) {
      printf("%2d:",i);
      switch (e_cfg->topmode) {
      default:
      case ETM_AS:
	ShowTopAS(stats->InBoundSrcAS,InBoundSrcAsTags,interval,stats->InBoundTotal);
	printf(" ");
	ShowTopAS(stats->InBoundDstAS,InBoundDstAsTags,interval,stats->InBoundTotal);
	break;
      case ETM_PROTO:
	ShowTopProto(stats->InBoundProto,InBoundProtoTags,interval,stats->InBoundTotal);
	break;
      case ETM_TCPPORT:
	ShowTopTcpPort(stats->InBoundSrcTcpPort,InBoundSrcTcpPortTags,
		       interval,stats->InBoundTotal);
	printf(" ");
	ShowTopTcpPort(stats->InBoundDstTcpPort,InBoundDstTcpPortTags,
		       interval,stats->InBoundTotal);
	break;
      case ETM_UDPPORT:
	ShowTopUdpPort(stats->InBoundSrcUdpPort,InBoundSrcUdpPortTags,
		       interval,stats->InBoundTotal);
	printf(" ");
	ShowTopUdpPort(stats->InBoundDstUdpPort,InBoundDstUdpPortTags,
		       interval,stats->InBoundTotal);
	break;
      }
      printf("\n");
    }
  }

  switch (e_cfg->topmode) {
  default:
  case ETM_AS:
    free(InBoundSrcAsTags);
    free(InBoundDstAsTags);
    free(OutBoundSrcAsTags);
    free(OutBoundDstAsTags);
    break;
  case ETM_PROTO:
    free(InBoundProtoTags);
    free(OutBoundProtoTags);
    break;
  case ETM_TCPPORT:
    free(InBoundSrcTcpPortTags);
    free(InBoundDstTcpPortTags);
    free(OutBoundSrcTcpPortTags);
    free(OutBoundDstTcpPortTags);
    break;
  case ETM_UDPPORT:
    free(InBoundSrcUdpPortTags);
    free(InBoundDstUdpPortTags);
    free(OutBoundSrcUdpPortTags);
    free(OutBoundDstUdpPortTags);
    break;
  }

}

static char str[100];

char * prettybytes(int bytes) {
  
  if (bytes < 1024) sprintf(str,"%4d", bytes);
  else if (bytes < (1024*1024)) sprintf(str,"%4dK",(bytes/1024));
  else if (bytes < (1024*1024*1024) ) sprintf(str,"%4dM",bytes/(1024*1024)); 
  /* apparantly 1024*1024*1024*1024 is too big for C */
  /*  else if (bytes < (double)(1024*1024*1024*1024) ) sprintf(str,"%3dG",bytes/(1024*1024*1024*1024)); */
  /*  else if (bytes < (1024*1024*1024*1024*1024) ) sprintf(str,"%3dT",bytes/(1024*1024*1024*1024*1024)); */

 return str;
}

double ShowTopAS (double a[],char t[], int interval, double total) {
  int i=-1,maxi=0;
  struct protoent *p;

  maxi=GetTop(a,t);

  if (a[maxi] == 0) return;
  if (maxi < 0) return;
  ShowValue(ASN_Lookup(maxi),maxi,a[maxi],interval,total);
}

double ShowTopProto (double a[],char t[], int interval, double total) {
  int i=-1,maxi=0;
  struct protoent *p;

  maxi=GetTop(a,t);

  if (a[maxi] == 0) return;

  p=getprotobynumber(maxi);
  if (p == NULL) {
    ShowValue("unknown",maxi,a[maxi],interval,total);
  } else {
    ShowValue(p->p_name,maxi,a[maxi],interval,total);
  }
}

double ShowTopTcpPort (double a[],char t[], int interval, double total) {
  int i=-1,maxi=0;
  struct servent *s;

  maxi=GetTop(a,t);

  if (a[maxi] == 0) return;

  s=getservbyport(htons(maxi),"tcp");
  if (s == NULL) {
    ShowValue("unknown",maxi,a[maxi],interval,total);
  } else {
    ShowValue(s->s_name,maxi,a[maxi],interval,total);
  }
}

double ShowTopUdpPort (double a[],char t[], int interval, double total) {
  int i=-1,maxi=0;
  struct servent *s;

  maxi=GetTop(a,t);

  if (a[maxi] == 0) return;

  s=getservbyport(htons(maxi),"udp");
  if (s == NULL) {
    ShowValue("unknown",maxi,a[maxi],interval,total);
  } else {
    ShowValue(s->s_name,maxi,a[maxi],interval,total);
  }
}

double GetTop (double a[], char t[]) {
  int i,maxi=0;
  double max=0;

  /* a quick, super-crappy scan for the highest number*/
  for (i=0; i<65536; i++) {
    if ( (a[i] > max) && (t[i] != 1) ) {
      max=a[i];
      maxi=i;
    }
  }

  if (t[maxi] == 1) { return -1; }

  if (max > 0) t[maxi]=1;

  return maxi;
}

int ShowValue(char * name,int maxi, double val, int interval, double total) {
  float percent;

  if (maxi >= 0) percent=((val/total)*100);
  else percent=0;

  printf(" %13.13s(%5d)",name,maxi);

  if ( (val*8/interval) >= 10240) {
    printf(" %6.0fk; %6.2f%%",((val*8)/interval)/1024,percent );
  } else {
    printf(" %6.0fb; %6.2f%%",((val*8)/interval),percent);
  }
}

int ShowReportTimeHeader(struct ehnt_stats * stats) {
  struct tm *tmp_tm;
  int in,h,m,s;
  char * timestr;
  
  in=stats->interval;
  s=in%60; in-=s;
  m=(in%3600)/60; in-=m*60;
  h=in/3600;
  
  tmp_tm=localtime(&stats->time);
  timestr=asctime(tmp_tm);
  timestr[strlen(timestr)-1]=(char)NULL;
  
  printf("Summary for interval (%2.2d:%2.2d:%2.2d total) ending %s\n",h,m,s,timestr);
}
