/*****************************************************************************
* Copyright (C) 2000 Nik Weidenbacher
* nikw@martnet.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*****************************************************************************/

#include <string.h>
#include <signal.h>
#include "config.h"
#include "ehnt.h"

void Usage(char *);
unsigned int Power(unsigned int, unsigned int);

int main(int argc, char ** argv) {

  int c;
  extern char *optarg;
  extern int optind, opterr, optopt;

  /* These three structs are used to communicate all info between functions.
     They are big and ugly, and I love them.  They are defined in ehnt.h. */
  struct ehnt_config e_cfg;
  struct ehnt_filter e_flt;
  struct ehnt_data   e_dat;

  /* We actually reduce the three structs to one for function calling convenience */
  struct ehnt_struct my_ehnt_struct;
  my_ehnt_struct.cfg=&e_cfg;
  my_ehnt_struct.flt=&e_flt;
  my_ehnt_struct.dat=&e_dat;

  /* set all variables to 0 */
  memset(&e_cfg,0,sizeof(e_cfg));
  memset(&e_flt,0,sizeof(e_flt));
  memset(&e_dat,0,sizeof(e_dat));

  /* set up some non-zero initial values */
  e_cfg.mode=EM_DUMP;
  e_cfg.si=60;
  memset(e_cfg.server,0,sizeof(e_cfg.server));
  e_cfg.zero_as=DEFAULT_ZERO_AS;
  sprintf(e_cfg.servername,DEFAULT_SERVER_IPADDR);
  e_cfg.serverport=DEFAULT_TCP_PORT;
  e_cfg.screenlength=DEFAULT_SCREENLENGTH;

  e_dat.stats = malloc(sizeof(struct ehnt_stats));

  signal(SIGPIPE,SIG_IGN);
  signal(SIGWINCH,SIG_IGN);

  while( (c=getopt(argc,argv,"0:a:bc:hi:l:m:n:p:P:r:s:t:vx:")) != -1) {

    opterr=0;
    switch(c) {
    case '0':
      sscanf(optarg,"%d",&e_cfg.zero_as);
      break;
    case 'a':
      sscanf(optarg,"%d",&e_flt.AS);
      printf("Filtering for AS %d\n",e_flt.AS);
      break;
    case 'b':
      e_flt.big=1;
      e_dat.maxbytes=0;
      e_dat.maxpackets=0;
      break;
    case 'c':
      sscanf(optarg,"%d",&e_cfg.flowlimit);
      break;
    case 'h':
      Usage(argv[0]);
      exit(0);
      break;
    case 'i':
      sscanf(optarg,"%d",&e_cfg.si);
      printf("Setting report interval to %d\n",e_cfg.si);
      break;
    case 'l':
      sscanf(optarg,"%d",&e_cfg.screenlength);
      printf("Setting screen length to %d lines\n",e_cfg.screenlength);
      break;
    case 'm':
      { 
	char em[100];
	sscanf(optarg,"%s",&em);
	if ( strcmp(em,"shortdump") == 0) {
	  e_cfg.mode=EM_SHORTDUMP;
	} else if ( strcmp(em,"colondump") == 0) {
	  e_cfg.mode=EM_COLONDUMP;
	} else if ( strcmp(em,"dump") == 0) {
	  e_cfg.mode=EM_DUMP;
	} else if ( strcmp(em,"top") == 0) {
	  e_cfg.mode=EM_TOP;
	} else {
	  printf("Mode %s not supported.  Available modes:\n",em);
	  printf(" dump\n shortdump\n colondump\n top\n");
	  exit(0);
	}
      }
      break;
    case 'n':
      if (! e_flt.router) {
	printf("Error: must specify a router with -r before using -e.\n");
	exit(0);
      }
      sscanf(optarg,"%d",&e_flt.intnum);
      printf("Displaying stats only for interface index %d\n",e_flt.intnum);
      break;
    case 'p':
      sscanf(optarg,"%d",&e_flt.port);
      printf("Filtering for port %d\n",e_flt.port);
      break;
    case 'P':
      sscanf(optarg,"%d",&e_flt.proto);
      printf("Filtering for protocol %d\n",e_flt.proto);
      break;
    case 'r':
      {
	char router[100];
	sscanf(optarg,"%100s",&router);
	e_flt.router=malloc(sizeof(struct in_addr));
	if (inet_aton(router,e_flt.router) == 0) {
	  printf("Error: invalid router address %s\n",router);
	  exit(1);
	}
      printf("Filtering for router %s\n",router);
      }
      break;
    case 's':
      { char * strptr;
	sscanf(optarg,"%100s",&e_cfg.server);
	if ( (strptr=strtok(e_cfg.server,":")) == NULL) {
	  printf("strtok could not find a server name in '%s'\n",e_cfg.server);
	  exit(0);
	}
	sprintf(e_cfg.servername,"%s",strptr);
	if ( (strptr=strtok(NULL,":")) == NULL) {
	  printf("strtok could not find a port number in '%s'\n",e_cfg.server);
	  exit(0);
	}
	sscanf(strptr,"%d",&e_cfg.serverport);
      }
      break;
    case 't':
      { 
	char etm[100];
	sscanf(optarg,"%s",&etm);
	if ( strcmp(etm,"as") == 0) {
	  e_cfg.topmode=ETM_AS;
	} else if ( strcmp(etm,"proto") == 0) {
	  e_cfg.topmode=ETM_PROTO;
	} else if ( strcmp(etm,"tcpport") == 0) {
	  e_cfg.topmode=ETM_TCPPORT;
	} else if ( strcmp(etm,"udpport") == 0) {
	  e_cfg.topmode=ETM_UDPPORT;
	} else {
	  printf("Top mode %s not supported.  Available top modes:\n",etm);
	  printf(" as\n proto\n tcpport\n udpport\n");
	  exit(0);
	}
      }
      break;
    case 'v':
      printf(EHNT_VERSION);
      printf("\n");
      exit(0);
      break;
    case 'x':
      { /* filter for prefix */
	char prefix[100];
	int pfxlen;
	sscanf(optarg,"%100[0123456789.]/%d\n",&prefix,&pfxlen);
	/* just store the prefix address in e_f/lt */
	e_flt.prefix=malloc(sizeof(struct in_addr));
	if (inet_aton(prefix,e_flt.prefix) == 0) {
	  printf("Error: invalid prefix %s\n",prefix);
	  exit(1);
	}
	/* compute the prefix mask and store it in e_flt */
	e_flt.pfxmask=malloc(sizeof(struct in_addr));
	if (pfxlen == 32) {
	  e_flt.pfxmask->s_addr=Power(2,32)-1;
	} else {
	  e_flt.pfxmask->s_addr=Power(2,32) - Power(2,32-pfxlen-1);
	}
	e_flt.pfxmask->s_addr=htonl(e_flt.pfxmask->s_addr);
	/* apply the mask to the prefix */
	e_flt.prefix->s_addr=e_flt.prefix->s_addr & e_flt.pfxmask->s_addr;
	printf("Filtering for prefix %s/%d\n",prefix,pfxlen);
      }
      break;
    default:
      Usage(argv[0]);
      exit(0);
    }

  }

  if ( (e_cfg.topmode > 0) && (e_cfg.mode != EM_TOP) ) {
    printf("Warning: topmode set but mode not set to top\n");
  }
  if ( (e_cfg.topmode == 0) && (e_cfg.mode == EM_TOP) ) {
    printf("topmode not set with -t, defaulting to as.\n");
    e_cfg.topmode=ETM_AS;
  }

  e_dat.last_t=time(NULL);
  
  {
    time_t mr_t;
    struct tm *ts;
    mr_t=time(NULL);
    ts=localtime(&mr_t);
    e_dat.recentmin=(int)(ts->tm_min + (ts->tm_hour*60) + (ts->tm_yday * 60 * 24));
  }

  NewStats(e_dat.stats);

  Init_ASN_Lookups();

  /* ehnt_client (in ehnt_client.c) connects to the server, then loops forever, 
     calling ProcessPacket for each incoming flow. */
  printf("Using report interval of %d minute(s)\n",e_cfg.si);
  printf("Starting. Hit '?' for help with keys.\r");

  ehnt_client(e_cfg.servername,e_cfg.serverport,&my_ehnt_struct);
}

void Usage(char *c) {
      printf("Usage: %s [ [option 1]  [option 2] ... [option n] ]\n",c);
      printf("Options:\n");
      printf("  -0 <ASN>      Replace AS number 0 occurences with this AS number\n");
      printf("  -a <ASN>      Only display flows to/from this AS number\n");
      printf("  -b            Display big flows (only shows flows with the most bytes\n");
      printf("                or packets received so far)\n");
      printf("  -c <count>    Exit after <count> flows are received.\n");
      printf("  -h            Display the cruft you are reading.\n");
      printf("  -i <interval> How long to wait between report generations (in minutes)\n");
      printf("  -l <lines>    The length of your display in lines.  Defaults to 24. Usa a\n");
      printf("                larger number if you want top mode to display more lines.\n");
      printf("  -m <mode>     The name of the mode of operation to use:\n");
      printf("                - 'dump' display flow detail\n");
      printf("                - 'shortdump' flow details in a more compact fashion\n");
      printf("                - 'colondump' flow details in somewhat machine-readable fashion\n");
      printf("                - 'top' generates reports of top average utilization\n");
      printf("  -n <intnum>   Specify the interface by SNMP ifIndex number\n");
      printf("  -p <port>     Only display flows to/from this tcp or udp port number\n");
      printf("  -P <proto>    Only display flows using this IP protocol number\n");
      printf("  -r <router>   Only display flows reported by this router IP address\n");
      printf("  -s <server:port>\n");
      printf("                The hostname or IP address and port number of the ehnt server\n");
      printf("  -t <topmode>  The type of report to generate when in top mode:\n");
      printf("                - 'as' AS report\n");
      printf("                - 'proto' Protocol report\n");
      printf("                - 'tcpport' TCP port report\n");
      printf("                - 'udpport' UDP port report\n");
      printf("  -v            Display version number\n");
      printf("  -x <prefix>   Only display flows to/from this IP prefix\n");
      printf("                The format for <prefix> is 'address/length', for example\n");
      printf("                1.2.3.4/30 or 10.0.0.0/8\n");
}

unsigned int Power(unsigned int x, unsigned int y) {
  int i;
  unsigned int r=x;

  for (i=1; i<=y; i++) {
    r=r*x;
  }

  return r;

}

